import { HttpClient } from '@angular/common/http'
import { Injectable } from '@angular/core'
import { Observable } from 'rxjs'
import { Collection } from '../viya-api-explorer/models/collection.model'
import { AppStoreService } from './app-store.service'
import { ViyaApis } from '../viya-api-explorer/models/viya-apis.models'
import { ViyaApiFolderMembers } from '../viya-api-explorer/models/viya-api-folder-content.model'
import { ViyaApiFolder } from '../viya-api-explorer/models/viya-api-folder.model'
import { ViyaApiIdentities } from '../viya-api-explorer/models/viya-api-identities.model'

@Injectable({
  providedIn: 'root'
})
export class SasViyaService {
  // Viya APIs
  public viyaApis: ViyaApis = {
    Analytics_Insights: {
      insights: '/insights'
    },
    Visualisation: {
      reports: '/reports',
      reportImages: '/reportImages',
      reportTransforms: '/reportTransforms',
      visualAnalytics: '/visualAnalytics'
    },
    Compute: {
      jobs: '/jobDefinitions',
      jobExecution: '/jobExecution',
      contexts: '/compute/contexts'
    },
    Decision_Management: {
      modelManagement: '/modelManagement',
      modelRepository: '/modelRepository',
      modelPublish: '/modelPublish',
      microanalyticScore: '/microanalyticScore',
      dataMining: '/dataMining',
      businessRules: '/businessRules',
      referenceData: '/referenceData',
      treatmentDefinitions: '/treatmentDefinitions',
      subjectContacts: '/subjectContacts',
      decisionsRuntimeBuilder: '/decisionsRuntimeBuilder'
    },
    Core_Services: {
      folders: '/folders',
      files: '/files',
      annotations: '/annotations',
      authorization: '/authorization',
      relationships: '/relationships',
      SASLogon: '/SASLogon'
    },
    Automated_Machine_Learning: {
      mlPipelineAutomation: '/mlPipelineAutomation'
    },
    Other: {
      identities: '/identities'
    }
  }

  private serverUrl: string = ''

  constructor(
    private http: HttpClient,
    private appStoreService: AppStoreService
  ) {}

  /**
   * This function is replacing the constructor.
   * The reason for this is timing issues, other services eg. sas.service, app-store.service
   * must be initialized before this bit of code is executed.
   * This function is being called by `sas.service`
   */
  setup() {
    const adapterConfig = this.appStoreService.getDcAdapterSettings()

    this.serverUrl = adapterConfig?.serverUrl || ''

    //example collection request
    // this.getByCollection('jobs').subscribe((res) => {
    //   console.log('res', res)
    // })
  }

  /**
   *
   * @returns All Viya API collections that we support
   */
  getAllCollections() {
    return this.viyaApis
  }

  /**
   * Used for clicking trough collections / apis
   * @param url collection/api endpoint url
   * @returns
   */
  getByUrl(url: string): Observable<Collection> {
    return this.http.get<Collection>(`${this.serverUrl}${url}`, {
      withCredentials: true
    })
  }

  /**
   * We refer collection as starting point /jobs /folders etc.
   * @param apiCollection
   * @returns
   */
  getByCollection(apiCollection: string): Observable<Collection> {
    return this.http.get<Collection>(`${this.serverUrl}${apiCollection}`, {
      withCredentials: true
    })
  }

  getComputeContexts(): Observable<any> {
    return this.http.get<any>(`${this.serverUrl}/compute/contexts`, {
      withCredentials: true
    })
  }

  /**
   * @param path Path to the folder
   * @returns The folder info object
   */
  getFolderByPath(path: string): Observable<ViyaApiFolder> {
    return this.http.get<any>(
      `${this.serverUrl}/folders/folders/@item?path=${path}`,
      {
        withCredentials: true
      }
    )
  }

  getFolderMembers(folderId: string): Observable<ViyaApiFolderMembers> {
    return this.http.get<any>(
      `${this.serverUrl}/folders/folders/${folderId}/members`,
      {
        withCredentials: true
      }
    )
  }

  getAdminGroups(limit: number = 5000): Observable<ViyaApiIdentities> {
    return this.http.get<any>(
      `${this.serverUrl}/identities/groups?sortBy=name&limit=${limit}`,
      {
        withCredentials: true
      }
    )
  }
}
